unit ApiToolTip;

{ Tooltip Class Unit

 Based on example by A. Kubaszek  <a_kubaszek@poczta.onet.pl>
 Based on "Using ToolTip Controls" in Win32 Programmer's Reference.

 (c) A. Kubaszek  <a_kubaszek@poczta.onet.pl>
 (c) Danger <danger@host.kz>

}

interface

uses Windows, Messages;

const
  TTDT_AUTOMATIC = 0;
  TTDT_RESHOW = 1;
  TTDT_AUTOPOP = 2;
  TTDT_INITIAL = 3;

  ICC_WIN95_CLASSES = $000000FF;
  cctrl = 'comctl32.dll';
  TOOLTIPS_CLASS = 'tooltips_class32';

  TTS_ALWAYSTIP = $01;
  TTS_NOPREFIX  = $02;
  TTF_SUBCLASS  = $0010;
  TTF_IDISHWND  = $0001;

  TTM_ACTIVATE         = WM_USER + 1;
  TTM_SETDELAYTIME     = WM_USER + 3;
  TTM_ADDTOOL          = WM_USER + 4;
  TTM_DELTOOL          = WM_USER + 5;
  TTM_NEWTOOLRECT      = WM_USER + 6;
  TTM_UPDATETIPTEXT    = WM_USER + 12;
  TTM_GETTOOLCOUNT     = WM_USER + 13;
  TTM_SETTIPTEXTCOLOR  = WM_USER + 20;


type
 // TTDT_Type
 TTDT_Type = (tdt_Automatic, tdt_Initial, tdt_Reshow, tdt_AutoPop);

 // TTooltip Class
 TToolTip = class (TObject)

  private
   fTThwnd: HWND;
   lf: tagLOGFONT;

   function GetToolCount: Integer;

  public      
   constructor Create(hWndParent, hInstance: HWND);
   destructor Destroy; override;

   function  AddTool(hwndTool: HWND; const Hint: string): Boolean;
   procedure UpdateTool(hwndTool: HWND; const NewHint: string);
   procedure NewRectTool(hwndTool: HWND; NewRect: TRect);
   procedure ActivateTool(hwndTool: HWND; fActivate: Boolean);
   procedure DeleteTool(hwndTool: HWND);

   procedure SetDelayTime(Time: TTDT_Type; Value: Integer);
   procedure SetFont(FName: string; FSize: Integer; FColor: Integer);

   property Handle: HWND read fTThwnd;
   property ToolCount: Integer read GetToolCount;
 end;

  // TInitCommonControlsEx / PInitCommonControlEx
  PInitCommonControlsEx = ^TInitCommonControlsEx;
  TInitCommonControlsEx = packed record
   dwSize: DWORD;
   dwICC: DWORD;
  end;

  // TToolInfo Record
  TToolInfo = packed record
   cbSize: UINT;
   uFlags: UINT;
   hwnd: HWND;
   uId: UINT;
   Rect: TRect;
   hInst: THandle;
   lpszText: PAnsiChar;
   lParam: LPARAM;
  end;

var ComCtl32_Module: HModule;

implementation

// From KOL Library

//[API InitCommonControls]
procedure InitCommonControls; external cctrl name 'InitCommonControls';

//[procedure DoInitCommonControls]
procedure DoInitCommonControls( dwICC: DWORD );
var Proc: procedure( ICC: PInitCommonControlsEx ); stdcall;
    ICC: TInitCommonControlsEx;

begin
  InitCommonControls;
  if ComCtl32_Module = 0 then
    ComCtl32_Module := LoadLibrary( 'comctl32.dll' );
  @ Proc := GetProcAddress( ComCtl32_Module, 'InitCommonControlsEx' );
  if Assigned( Proc ) then
  begin
    ICC.dwSize := Sizeof( ICC );
    ICC.dwICC := dwICC;
    Proc( @ ICC );
  end;
end;

//[END DoInitCommonControls]

//-----------------------------------------------------

constructor TToolTip.Create(hWndParent, hInstance: HWND);
begin

 //see MSDN: "Using ToolTip Controls", CreateMyTooltip()
 DoInitCommonControls( ICC_WIN95_CLASSES );
 // To ensure that the common control DLL is loaded

 // CREATE A TOOLTIP WINDOW
 fTThWnd := CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, nil,
             WS_POPUP or TTS_NOPREFIX or TTS_ALWAYSTIP,
             CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
             hWndParent, 0, hInstance, nil );

 SetWindowPos(fTThWnd, HWND_TOPMOST, 0, 0, 0, 0,
      SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE);
end;

//-----------------------------------------------------

destructor TToolTip.Destroy;
begin
  DestroyWindow(fTThWnd);
  inherited;
end;

//-----------------------------------------------------

function TToolTip.AddTool(hwndTool: HWND; const Hint: string): Boolean;
var
 ti: TTOOLINFO; // struct specifying info about tool in ToolTip control

begin
  // INITIALIZE MEMBERS OF THE TOOLINFO STRUCTURE
  ti.cbSize:= SizeOf(TTOOLINFO);
  ti.uFlags:= TTF_SUBCLASS or TTF_IDISHWND;
  ti.hwnd:= hwndTool;
  ti.hInst:= 0;
  ti.uId:= hwndTool;
  ti.lpszText:= PChar(Hint);

 Result:= BOOL(SendMessage(fTThwnd, TTM_ADDTOOL, 0, LPARAM(@ti)));
end;

//-----------------------------------------------------

procedure TToolTip.DeleteTool(hwndTool: HWND);
var
 ti: TTOOLINFO; // struct specifying info about tool in ToolTip control

begin
 // INITIALIZE MEMBERS OF THE TOOLINFO STRUCTURE
  ti.cbSize:= SizeOf(TTOOLINFO);
  ti.hwnd:= hwndTool;
  ti.hInst:= 0;
  ti.uId:= hwndTool;

  SendMessage(fTThwnd, TTM_DELTOOL, 0, LPARAM(@ti));
end;

//-----------------------------------------------------

procedure TToolTip.SetDelayTime(Time: TTDT_Type; Value: Integer);
begin
 case Time of
   tdt_Automatic: SendMessage(fTThwnd, TTM_SETDELAYTIME, TTDT_AUTOMATIC, Value);
   tdt_Initial: SendMessage(fTThwnd, TTM_SETDELAYTIME, TTDT_INITIAL, Value);
   tdt_Reshow: SendMessage(fTThwnd, TTM_SETDELAYTIME, TTDT_RESHOW, Value);
   tdt_AutoPop: SendMessage(fTThwnd, TTM_SETDELAYTIME, TTDT_AUTOPOP, Value);
 end;

end;

//-----------------------------------------------------

procedure TToolTip.SetFont(FName: string; FSize: Integer; FColor: Integer);
begin
 ZeroMemory(@lf, SizeOf(lf));
 lf.lfHeight:= FSize;
 lf.lfWeight:= FW_NORMAL;
 lf.lfCharSet:= DEFAULT_CHARSET;
 lf.lfOutPrecision:= OUT_DEFAULT_PRECIS;
 lf.lfClipPrecision:= CLIP_DEFAULT_PRECIS;
 lf.lfQuality:= DEFAULT_QUALITY;
 lf.lfPitchAndFamily:= FF_DONTCARE + DEFAULT_PITCH;

 // Font Name may not contain more than 32 chars [!]
 if Length(FName) > 32 then Exit;
  Move(PChar(FName)^, lf.lfFaceName, Length(FName));

 SendMessage(fTThwnd, WM_SETFONT, CreateFontIndirect(lf), 0);
 SendMessage(fTThwnd, TTM_SETTIPTEXTCOLOR, FColor, 0);

end;

//-----------------------------------------------------

function TToolTip.GetToolCount: Integer;
begin
  Result:= SendMessage(fTThwnd, TTM_GETTOOLCOUNT, 0, 0);
end;

//-----------------------------------------------------

procedure TToolTip.UpdateTool(hwndTool: HWND; const NewHint: string);
var
 ti: TTOOLINFO; // struct specifying info about tool in ToolTip control

begin
  // INITIALIZE MEMBERS OF THE TOOLINFO STRUCTURE
  ti.cbSize:= SizeOf(TTOOLINFO);
  ti.hwnd:= hwndTool;
  ti.hInst:= 0;
  ti.uId:= hwndTool;
  ti.lpszText:= PChar(NewHint);

  SendMessage(fTThwnd, TTM_UPDATETIPTEXT, 0, LPARAM(@ti));
end;

//-----------------------------------------------------

procedure TToolTip.NewRectTool(hwndTool: HWND; NewRect: TRect);
var
 ti: TTOOLINFO; // struct specifying info about tool in ToolTip control
begin
   // INITIALIZE MEMBERS OF THE TOOLINFO STRUCTURE
  ti.cbSize:= SizeOf(TTOOLINFO);
  ti.hwnd:= hwndTool;
  ti.hInst:= 0;
  ti.uId:= hwndTool;
  ti.Rect:= NewRect;

  SendMessage(fTThwnd, TTM_NEWTOOLRECT, 0, LPARAM(@ti));
end;

//-----------------------------------------------------

procedure TToolTip.ActivateTool(hwndTool: HWND; fActivate: Boolean);
begin
 SendMessage(fTThwnd, TTM_ACTIVATE, WPARAM(fActivate), 0);
end;

//-----------------------------------------------------

end.
